package it.unibo.tw0708.spring.main;

import it.unibo.tw0708.beans.compactDiscs.ICompactDiscsManager;
import it.unibo.tw0708.beans.user.IUserManager;
import it.unibo.tw0708.beans.user.NonInterfaceBasedUserManager;
import it.unibo.tw0708.beans.user.UserManager;
import it.unibo.tw0708.dao.CompactDiscTO;

import java.util.Collection;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.support.ClassPathXmlApplicationContext;

public class ApplicationContextLoaderMainClass {

	public static void main(String[] args)
	throws Exception {
		
		// starts up the application context
		BeanFactory applicationContext = new ClassPathXmlApplicationContext("appMainApplicationContext.xml");
		
		// programmatically gets some bean proxies
		ICompactDiscsManager compactDiscsManager = (ICompactDiscsManager) applicationContext.getBean("compactDiscsManager");
		// achieves a proxy of dependening-on bean
		UserManager userManager = (UserManager) compactDiscsManager.getOwner();
		// and the logger to print out something interesting...
		Logger rootLogger = (Logger) applicationContext.getBean("logger");

		// check beans' scope-driven behaviour
		//
		// this fails, 'cause cd manager is prototype and Java interface-based proxied: proxy creates a new instance of it upon every request!
		compactDiscsManager.setSomeProperty("aValue");					
		rootLogger.info("\n" + "compactDiscsManager has someProperty: " + compactDiscsManager.getSomeProperty() + " (correct is: null)\n");
		//
		// this works, since userManager is session-alike scoped (see the XML descriptor)!
		userManager.setName("aUser");					
		rootLogger.info("\n" + "userManager holds user name: " + userManager.getName() + " (correct is: aUser)\n");

		// now some tricks about the custom scopes that simulate session outside of the http server
		userManager.setName("anotherUser");
		userManager = (UserManager) applicationContext.getBean("userManager");
		rootLogger.info("\n" + "userManager holds user name: " + userManager.getName() + " (correct is: anotherUser)\n");
		// custom non-http-session scope is class loader driven: as long as we have the same class loader we get the same bean
		Thread.currentThread().setContextClassLoader(new CustomClassLoader());
		userManager = (UserManager) applicationContext.getBean("userManager");
		rootLogger.info("\n" + "userManager holds user name: " + userManager.getName() + " (correct is: unknownNonHttpUser)\n");

		// now tips and tricks about Java-interface vs. CGLIB proxying
		// xml descriptors have been configured to achieve stateful or stateless behaviour
		IUserManager conversationalInterfaceBasedUserManager = (IUserManager) applicationContext.getBean("conversationalInterfaceBasedUserManager");
		IUserManager nonConversationalInterfaceBasedUserManager = (IUserManager) applicationContext.getBean("nonConversationalInterfaceBasedUserManager");
		NonInterfaceBasedUserManager conversationalNonInterfaceBasedUserManager = (NonInterfaceBasedUserManager) applicationContext.getBean("conversationalNonInterfaceBasedUserManager");
		//
		conversationalInterfaceBasedUserManager.setName("conversationalInterfaceBasedUser");
		conversationalNonInterfaceBasedUserManager.setName("conversationalNonInterfaceBasedUser");
		nonConversationalInterfaceBasedUserManager.setName("nonConversationalInterfaceBasedUser");
		//
		rootLogger.info("\n" + "conversationalInterfaceBasedUserManager holds user name: " + conversationalInterfaceBasedUserManager.getName() + " (correct is: conversationalInterfaceBasedUser)\n");
		rootLogger.info("\n" + "conversationalNonInterfaceBasedUserManager holds user name: " + conversationalNonInterfaceBasedUserManager.getName() + " (correct is: conversationalNonInterfaceBasedUser)\n");
		rootLogger.info("\n" + "nonConversationalInterfaceBasedUserManager holds user name: " + nonConversationalInterfaceBasedUserManager.getName() + " (correct is: anotherUnknownUser)\n");

		// check aspect-based tracing
		userManager.load();
		conversationalInterfaceBasedUserManager.load();
		conversationalNonInterfaceBasedUserManager.load();
		nonConversationalInterfaceBasedUserManager.load();
		userManager.load();
		conversationalInterfaceBasedUserManager.load();
		conversationalNonInterfaceBasedUserManager.load();
		nonConversationalInterfaceBasedUserManager.load();
		rootLogger.info("\n" + "XXXuserManager.load() method should be traced by an aspect (correctly you should see some log around here...)\n");

		// check aspect-based dao exploitation
		compactDiscsManager.getOwner().setName("pisi");
		int collectionSize = compactDiscsManager.getCollectionSize();
		rootLogger.info("\n" + "compactDiscsManager.getCollectionSize() method should be traced by an aspect (correctly you should see some log around here...)\n");
		String element_s = ( collectionSize == 1 ) ? "element" : "elements";
		rootLogger.info("\ncompactDiscsManager currently holds a CD collection of " + collectionSize + " " + element_s + " (correct is: 2)\n");
		Collection<CompactDiscTO> compactDiscsCollection = compactDiscsManager.getCompactDiscs();
		if ( compactDiscsCollection != null && compactDiscsCollection.size() > 0 )
			for ( CompactDiscTO compactDisc : compactDiscsCollection )
				rootLogger.info("\nuser " + compactDiscsManager.getOwner().getName() + " owns compact disc: " + compactDisc.toString() + "\n");

	}

	// -------------------------------------------------
	
	private static class CustomClassLoader extends ClassLoader {
		public CustomClassLoader() { super(); }
		public CustomClassLoader(ClassLoader arg0) { super(arg0); }
	}

}
